import React from 'react';
import ReactDOM from 'react-dom';
import TestUtils from 'react-addons-test-utils';
import purebem from 'purebem';

import Dropdown from './Dropdown';

import dropdownValueOptionState from './option-value/dropdownValueOptionState';


const TestDropdown = Dropdown({
    block: purebem.of('test-dropdown'),
    displayName: 'TestDropdown'
});

const getOptions = (a, b, c) => {
    return [
        dropdownValueOptionState({
            disabled: a,
            text: 'A',
            value: 'a'
        }),
        dropdownValueOptionState({
            disabled: b,
            text: 'B',
            value: 'b'
        }),
        dropdownValueOptionState({
            disabled: c,
            text: 'C',
            value: 'c'
        })
    ];
};

describe('Dropdown', () => {

    [
        { name: 'should expand collapsed dropdown when using enter', data: { which: 13 } },
        { name: 'should expand collapsed dropdown when using space', data: { which: 32 } },
        { name: 'should expand collapsed dropdown when using down arrow', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            let expanded = false;

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown expanded={ expanded } onExpand={ () => expanded = true } />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(expanded).toBeTruthy();
        });
    });

    it('should collapse expanded dropdown when using escape', () => {
        let expanded = true;

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown expanded={ expanded } onCollapse={ () => expanded = false } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 27 });

        expect(expanded).toBeFalsy();
    });

    [
        { name: 'should collapse empty expanded dropdown when using enter', data: { which: 13 } },
        { name: 'should collapse empty expanded dropdown when using space', data: { which: 32 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            let expanded = true;

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown expanded={ expanded } onCollapse={ () => expanded = false } options={ [] } />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(expanded).toBeFalsy();
        });
    });

    [
        { name: 'should change value when using enter on new option', data: { which: 13 } },
        { name: 'should change value when using space on new option', data: { which: 32 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            let value = 'a';

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown
                    expanded={ true }
                    highlightedIndex={ 1 }
                    onChange={ (v) => value = v }
                    options={ getOptions(false, false, false) }
                    value={ value } />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(value).toBe('b');
        });
    });

    [
        { name: 'should cancel when using enter on same option', data: { which: 13 } },
        { name: 'should cancel when using space on same option', data: { which: 32 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            let wasCancelCalled = false;
            let wasChangeCalled = false;

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown
                    expanded={ true }
                    highlightedIndex={ 0 }
                    onChange={ () => wasChangeCalled = true }
                    onCollapse={ () => wasCancelCalled = true }
                    options={ getOptions(false, false, false) }
                    value="a" />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(wasCancelCalled).toBeTruthy();
            expect(wasChangeCalled).toBeFalsy();
        });
    });

    it('should change highlighted index when using down arrow when expanded', () => {
        let highlightedIndex = 0;

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ highlightedIndex }
                onHighlightedIndexChange={ (index) => highlightedIndex = index }
                options={ getOptions(false, false, false) } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 40 });

        expect(highlightedIndex).toBe(1);
    });

    it('should change highlighted index when using up arrow when expanded', () => {
        let highlightedIndex = 2;

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ highlightedIndex }
                onHighlightedIndexChange={ (index) => highlightedIndex = index }
                options={ getOptions(false, false, false) } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 38 });

        expect(highlightedIndex).toBe(1);
    });

    it('should skip disabled options when using down arrow when expanded', () => {
        let highlightedIndex = 0;

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ highlightedIndex }
                onHighlightedIndexChange={ (index) => highlightedIndex = index }
                options={ getOptions(false, true, false) } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 40 });

        expect(highlightedIndex).toBe(2);
    });

    it('should skip disabled options when using up arrow when expanded', () => {
        let highlightedIndex = 2;

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ highlightedIndex }
                onHighlightedIndexChange={ (index) => highlightedIndex = index }
                options={ getOptions(false, true, false) } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 38 });

        expect(highlightedIndex).toBe(0);
    });

    it('should wrap around highligthed index when using down arrow when expanded and on last option', () => {
        let highlightedIndex = 2;

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ highlightedIndex }
                onHighlightedIndexChange={ (index) => highlightedIndex = index }
                options={ getOptions(false, false, false) } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 40 });

        expect(highlightedIndex).toBe(0);
    });

    it('should wrap around highlighted index when using up arrow when expanded and on first option ', () => {
        let highlightedIndex = 0;

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ highlightedIndex }
                onHighlightedIndexChange={ (index) => highlightedIndex = index }
                options={ getOptions(false, false, false) } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 38 });

        expect(highlightedIndex).toBe(2);
    });

    [
        { name: 'should ignore up arrow key when all options are disabled', data: { which: 38 } },
        { name: 'should ignore down arrow key when all options are disabled', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            let highlightedIndex = 0;

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown
                    expanded={ true }
                    highlightedIndex={ highlightedIndex }
                    onHighlightedIndexChange={ (index) => highlightedIndex = index }
                    options={ getOptions(true, true, true) } />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(highlightedIndex).toBe(0);
        });
    });

    it('should select first if no initial value when expanded and using down arrow ', () => {
        let highlightedIndex = -1;

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ highlightedIndex }
                onHighlightedIndexChange={ (index) => highlightedIndex = index }
                options={ getOptions(false, false, false) } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 40 });

        expect(highlightedIndex).toBe(0);
    });

    it('should select last if no initial value when expanded and using up arrow ', () => {
        let highlightedIndex = -1;

        const dropdown = TestUtils.renderIntoDocument(
            <TestDropdown
                expanded={ true }
                highlightedIndex={ highlightedIndex }
                onHighlightedIndexChange={ (index) => highlightedIndex = index }
                options={ getOptions(false, false, false) } />
        );

        const node = ReactDOM.findDOMNode(dropdown);
        TestUtils.Simulate.keyDown(node, { which: 38 });

        expect(highlightedIndex).toBe(2);
    });

    [
        { name: 'should ignore up arrow key when all options are disabled', data: { which: 38 } },
        { name: 'should ignore down arrow key when all options are disabled', data: { which: 40 } }
    ].forEach((fixture) => {
        it(fixture.name, () => {
            const highlightedIndex = -1;
            let wasCalled = false;

            const dropdown = TestUtils.renderIntoDocument(
                <TestDropdown
                    expanded={ true }
                    highlightedIndex={ highlightedIndex }
                    onHighlightedIndexChange={ () => wasCalled = true }
                    options={ [] } />
            );

            const node = ReactDOM.findDOMNode(dropdown);
            TestUtils.Simulate.keyDown(node, { which: fixture.data.which });

            expect(wasCalled).toBeFalsy();
        });
    });

});
